from typing import Callable, Dict, List
from http.server import BaseHTTPRequestHandler, HTTPServer
import json

class Whale:
    def __init__(self, name: str = "Whale"):
        self.name = name
        self.routes: Dict[str, Dict[str, Callable]] = {}
        self.middleware: List[Callable] = []

    def route(self, path: str, method: str = "GET"):
        def decorator(func: Callable):
            if path not in self.routes:
                self.routes[path] = {}
            self.routes[path][method.upper()] = func
            return func
        return decorator

    def use(self, middleware: Callable):
        self.middleware.append(middleware)

    def run(self, host: str = "127.0.0.1", port: int = 5000):
        class WhaleHandler(BaseHTTPRequestHandler):
            def do_GET(self):
                self.handle_request("GET")

            def do_POST(self):
                self.handle_request("POST")

            def handle_request(self, method: str):
                path = self.path
                if path in self.server.whale.routes and method in self.server.whale.routes[path]:
                    for middleware in self.server.whale.middleware:
                        middleware(self)
                    response = self.server.whale.routes[path][method]()
                    self.send_response(200)
                    self.send_header("Content-type", "application/json")
                    self.end_headers()
                    self.wfile.write(json.dumps(response).encode())
                else:
                    self.send_error(404, "Not Found")

        server = HTTPServer((host, port), WhaleHandler)
        server.whale = self
        print(f"Running {self.name} on http://{host}:{port}")
        server.serve_forever()